﻿using PuppeteerSharp;
using System;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

using var browser = await Puppeteer.LaunchAsync(new LaunchOptions
{
    Headless = true,
    SlowMo = 20,
    ExecutablePath = @"C:\Program Files (x86)\Microsoft\Edge\Application\msedge.exe"
});
var page = await browser.NewPageAsync();
var userAgent = await browser.GetUserAgentAsync();
await page.SetUserAgentAsync($"{userAgent} TestBot");

await page.GoToAsync("https://www.wikipedia.org/");

await page.SetzeSpracheImSuchfeld("es");

var navigationTask = page.WaitForNavigationAsync();
await page.GebeSuchBegriffEinUndStarteSuche("mallorca");
await navigationTask;

await page.WaitForSelectorAsync(".infobox");
string[] tableRowTexts = await page
    .EvaluateExpressionAsync<string[]>(@"Array.from(document.querySelector('.infobox').querySelectorAll('tr')).map(tr => tr.innerText)");

var populationRegex = new Regex(@"Total\s([\d\s]*)hab\.");
var populationText = FindFirstRegexInStringArray(tableRowTexts, populationRegex);

if (populationText != null)
{
    Console.WriteLine($"Laut der spanischen Wikipedia hat Mallorca {populationText.Trim()} Einwohner.");
}

static string? FindFirstRegexInStringArray(string[] paragraphs, Regex regex)
{
    foreach (string paragraph in paragraphs)
    {
        var regexMatch = regex.Match(paragraph);
        if (regexMatch.Success)
        {
            return regexMatch.Groups[1].ToString();
        }
    }
    return null;
}

public static class WikipediaPageExtensions
{
    public static async Task SetzeSpracheImSuchfeld(this Page page, string language)
        => await page.SelectAsync("#searchLanguage", language);

    public static async Task GebeSuchBegriffEinUndStarteSuche(this Page page, string searchstring)
    {
        var elementHandle = await page.QuerySelectorAsync("#searchInput");
        await elementHandle.TypeAsync(searchstring);
        await elementHandle.PressAsync(PuppeteerSharp.Input.Key.Enter);
    }
}